@@ -47,10 +47,17 @@ class Context(object):
                                from environment variables is disabled.  This
                                does not affect manually set environment
                                variables which are always read.
+    :param default_map: a dictionary (like object) with default values
+                        for parameters.
+    :param terminal_width: the width of the terminal.  The default is
+                           inherit from parent context.  If no context
+                           defines the terminal width then auto
+                           detection will be applied.
     """
 
     def __init__(self, command, parent=None, info_name=None, obj=None,
-                 auto_envvar_prefix=None):
+                 auto_envvar_prefix=None, default_map=None,
+                 terminal_width=None):
         #: the parent context or `None` if none exists.
         self.parent = parent
         #: the :class:`Command` for this context.
@@ -72,6 +79,13 @@ class Context(object):
             obj = parent.obj
         #: the user object stored.
         self.obj = obj
+        #: A dictionary (like object) with defaults for parameters.
+        self.default_map = default_map
+
+        if terminal_width is None and parent is not None:
+            terminal_width = parent.terminal_width
+        #: The width of the terminal (None is autodetection).
+        self.terminal_width = terminal_width
 
         # If there is no envvar prefix yet, but the parent has one and
         # the command on this level has a name, we can expand the envvar
@@ -94,6 +108,10 @@ class Context(object):
     def __exit__(self, exc_type, exc_value, tb):
         self.close()
 
+    def make_formatter(self):
+        """Creates the formatter for the help and usage output."""
+        return HelpFormatter(width=self.terminal_width)
+
     def call_on_close(self, f):
         """This decorator remembers a function as callback that should be
         executed when the context tears down.  This is most useful to bind
@@ -149,6 +167,16 @@ class Context(object):
             self.obj = rv = object_type()
         return rv
 
+    def lookup_default(self, name):
+        """Looks up the default for a parameter name.  This by default
+        looks into the :attr:`default_map` if available.
+        """
+        if self.default_map is not None:
+            rv = self.default_map.get(name)
+            if callable(rv):
+                rv = rv()
+            return rv
+
     def fail(self, message):
         """Aborts the execution of the program with a specific error
         message.
@@ -289,7 +317,7 @@ class Command(object):
 
         -   :meth:`format_usage`
         """
-        formatter = HelpFormatter()
+        formatter = ctx.make_formatter()
         self.format_usage(ctx, formatter)
         return formatter.getvalue().rstrip('\n')
 
@@ -302,7 +330,7 @@ class Command(object):
         -   :meth:`format_options`
         -   :meth:`format_epilog`
         """
-        formatter = HelpFormatter()
+        formatter = ctx.make_formatter()
         self.format_usage(ctx, formatter)
         self.format_help(ctx, formatter)
         self.format_options(ctx, formatter)
@@ -362,6 +390,11 @@ class Command(object):
         :param extra: extra keyword arguments forwarded to the context
                       constructor.
         """
+        if 'default_map' not in extra:
+            default_map = None
+            if parent is not None and parent.default_map is not None:
+                default_map = parent.default_map.get(info_name)
+            extra['default_map'] = default_map
         ctx = Context(self, info_name=info_name, parent=parent, **extra)
         parser = self.make_parser(ctx)
         opts, args = parser.parse_args(args=args)
@@ -400,7 +433,7 @@ class Command(object):
                           the progam name is constructed by taking the file
                           name from ``sys.argv[0]``.
         :param extra: extra keyword arguments are forwarded to the context
-                      constructor.
+                      constructor.  See :class:`Context` for more information.
         """
         # If we are on python 3 we will verify that the environment is
         # sane at this point of reject further execution to avoid a
@@ -701,6 +734,8 @@ class Parameter(object):
 
     def consume_value(self, ctx, opts):
         value = opts.get(self.name)
+        if value is None:
+            value = ctx.lookup_default(self.name)
         if value is None:
             value = self.value_from_envvar(ctx)
         return value
@@ -886,6 +921,8 @@ class Option(Parameter):
         if name is None and possible_names:
             possible_names.sort(key=len)
             name = possible_names[-1].replace('-', '_')
+            if not isidentifier(name):
+                name = None
 
         if name is None:
             raise TypeError('Could not determine name for option')
